<?php
declare(strict_types = 1);

namespace App\Transaction\Handlers;

use Apex\Svc\{App, Db, Convert};
use App\Transaction\Handlers\TxValidator;
use App\Transaction\Models\Tx;
use App\Transaction\Enums\{TxStatus, TxAmountType};

/**
 * Transaction handler
 */
class TxHandler
{

    #[Inject(App::class)]
    private App $app;

    #[Inject(Db::class)]
    private Db $db;

    #[Inject(Convert::class)]
    private Convert $convert;

    #[Inject(TxValidator::class)]
    private TxValidator $validator;

    /**
     * Create transaction
     */
    public function create(Tx $tx):?Tx
    {

        // Validate tx
        $tx = $this->validator->validate($tx);

        // Get amount
        $adapter = $tx->getAdapter();
        $tx->amount = $adapter->getAmount($tx);

        // Save to database
        $tx = Tx::insert($tx);

        // Update balance
        $this->updateBalance($tx);

        // Execute adapter method
        $this->executeAdapterStatusMethod($tx);

        // Return
        return $tx;
    }

    /**
     * Update balance
     */
    public function updateBalance(Tx $tx, ?Tx $old_tx = null):float | string
    {

        // Initialize
        $acct = $tx->getAccount();

        // Process old_tx as necessary
        if ($old_tx !== null && $old_tx->status == $tx->status) {
            return $acct->balance;
        } elseif ($old_tx !== null && in_array($old_tx->status, [TxStatus::APPROVED, TxStatus::PENDING, TxStatus::FROZEN])) {
            $column = $old_tx->status == TxStatus::APPROVED ? 'balance' : 'pending';
            if ($old_tx->status == TxStatus::FROZEN) {
                $column = 'frozen';
            }

            if (is_string($old_tx->amount)) {
                $acct->$column = bcsub((string) $acct->$column, $old_tx->amount, 8); 
            } else {
                $acct->$column -= $old_tx->amount;
            }
        }

        // Process new tx
        if (in_array($tx->status, [TxStatus::APPROVED, TxStatus::PENDING, TxStatus::FROZEN])) {
            $column = $tx->status == TxStatus::APPROVED ? 'balance' : 'pending';
            if ($tx->status == TxStatus::FROZEN) {
                $column = 'frozen';
            }

            if (is_string($tx->amount)) {
                $acct->$column = bcadd((string) $acct->$column, $tx->amount, 8);
            } else {
                $acct->$column += $tx->amount;
            }
        }

        // Return
        $acct->save();
        return $acct->balance;
    }

    /**
     * Update status
     */
    public function updateStatus(Tx $tx, TxStatus $new_status, bool $is_saving = false): Tx
    {

        // Check if status the same
        if ($tx->status == $new_status) {
            return $tx;
        }

        // Clone tx
        $new_tx = clone $tx;
        $new_tx->status = $new_status;

        // Update balance
        $this->updateBalance($new_tx, $tx);

        // Execute adapter method
        $this->executeAdapterStatusMethod($new_tx);

        // Save to database, if needed
        $this->db->query("UPDATE transaction SET status = %s WHERE id = %i", $new_status->value, $tx->id);

        // Check pending transactions, if needed
        if ($new_status == TxStatus::APPROVED) {
            $this->checkPending($new_tx);
        }

        // Return
        return $new_tx;
    }

    /**
     * Update amount
     */
    public function updateAmount(Tx $tx, float $new_amount, TxAmountType $type = TxAmountType::ACTUAL, bool $is_saving = false): Tx
    {

        // Check amounts are different
        $prop_name = $type->value;
        if ($tx->$prop_name == $new_amount) {
            return $tx;
        }

        // Clone tx
        $new_tx = clone $tx;
        $new_tx->$prop_name = $new_amount;

        // Update auxillary amounts
        $def_currency = $this->app->config('core.default_currency');
        if ($prop_name == 'base_amount') {
            $new_tx->amount = $this->convert->exchange_money($new_amount, $def_currency, $tx->currency, $tx->created_at);
            $new_tx->from_amount = $this->convert->exchange_money($new_amount, $def_currency, $tx->from_currency, $tx->created_at);
        } elseif ($prop_name == 'from_amount') {
            $new_tx->amount = $this->convert->exchange_money($new_amount, $tx->from_currency, $tx->currency, $tx->created_at);
            $new_tx->base_amount = $this->convert->exchange_money($new_amount, $tx->from_currency, $def_currency, $tx->created_at);
        } else {
            $new_tx->from_amount = $this->convert->exchange_money($new_amount, $tx->currency, $tx->from_currency, $tx->created_at);
            $new_tx->base_amount = $this->convert->exchange_money($new_amount, $tx->currency, $def_currency, $tx->created_at);
        }

        // Update balance, if needed
        if ($tx->status == TxStatus::APPROVED) {
            $diff_amount = ($new_tx->amount - $tx->amount);
            $this->db->query("UPDATE transaction_accounts SET balance = balance + %d WHERE id = %i", $diff_amount, $tx->account_id);
        }

        // Save to database, if needed
        if ($is_saving !== true) {
            $new_tx->save();
        }

        // Return
        return $new_tx;
    }

    /**
     * Execute adapter status method
     */
    private function executeAdapterStatusMethod(Tx $tx): void
    {

        // Get adapter
        $adapter = $tx->getAdapter();
        $status_str = $tx->status->value;

        // Execute method
        if (method_exists($adapter, $status_str)) {
            $adapter->$status_str($tx);
        }
    }

    /**
     * Check pending transactions if now paid
     */
    private function checkPending(Tx $tx):void
    {

        // Go through pending
        $rows = Tx::where("account_id = %i AND status = 'pending' AND amount < 0.00000000 ORDER BY id", $tx->account_id);
        foreach ($rows as $row) {

            // Check balnce
            $balance = (string) $this->db->getField("SELECT balance FROM transaction_accounts WHERE id = %i", $tx->account_id);
            $chk_amount = preg_replace("/^\-/", "", (string) $row->amount);
            if (bccomp($chk_amount, $balance, 8) == 1) {
                continue;
            }

            // Update status
            $this->updateStatus($row, TxStatus::APPROVED);
        }

    }

}

