<?php
declare(strict_types=1);

namespace App\Transaction\Handlers;

use Apex\Svc\{App, Convert};
use App\Transaction\Models\{Tx, TxAccount, Currency};
use App\Transaction\Handlers\TxAccountHandler;
use App\Transaction\Interfaces\AdapterInterface;
use App\Transaction\Exceptions\TxValidationException;

/**
 * Tx validator
 */
class TxValidator
{

    #[Inject(App::class)]
    private App $app;

    #[Inject(Convert::class)]
    private Convert $convert;

    #[Inject(TxAccountHandler::class)]
    private TxAccountHandler $acct_handler;

    /**
     * Validate
     */
    public function validate(Tx $tx):?Tx
    {

        // Check currency
        $tx = $this->checkCurrency($tx);

        // Check account id
        if ($tx->account_id == 0) {
            $tx = $this->checkAccountId($tx);
        }

        // Chek adapter
        $this->checkAdapter($tx);

        // Check amount
        $tx = $this->checkAmount($tx);

        // Check ip address
        if ($tx->ip_address == '') {
            $tx->ip_address = $this->app->getClient()->getIpAddress();
        }

        // Return
        return $tx;
    }

    /**
     * Check currency
     */
    public function checkCurrency(Tx $tx):Tx
    {

        // Currency
        if ($tx->currency == '') {
            $tx->currency = $this->app->config('core.default_currency');
        }

        // From currency
        if ($tx->from_currency == '') {
            $tx->from_currency = $this->app->config('core.default_currency');
        }

        // Return
        return $tx;
    }

    /**
     * Check account id
     */
    public function checkAccountId(Tx $tx):?Tx
    {

        // Check if we have account id
        if ($tx->account_id > 0) {
            return $tx;
        }

        // Ensure we have uuid
        if ($tx->uuid == '') {
            throw new TxValidationException("Unable to determine transaction account as no account id or user was defined within the transaction.");
        }

        // Lookup account
        $acct = $this->acct_handler->lookup($tx->uuid, $tx->currency);
        $tx->account_id = $acct->id;

        // Return
        return $tx;
    }

    /**
     * Check adapter
     */
    public function checkAdapter(Tx $tx):void
    {

        // Ensure class exists
        if (!class_exists($tx->adapter)) {
            throw new TxValidationException("Transaction adapter does not exist at, $tx->adapter");
        }

        // Check interface of class
        $obj = new \ReflectionClass($tx->adapter);
        if (!$obj->implementsInterface(AdapterInterface::class)) {
            throw new TxValidationException("Transaction adapter is invalid as it does not implement the AdapterInterface interface, $tx->adapter");
        }

    }

    /**
     * Check amount
     */
    public function checkAmount(Tx $tx): Tx
    {

        // Check for an amount
        if ($tx->amount == 0.00 && $tx->from_amount == 0.00) {
            throw new TxValidationException("No amount or from_amount specified with the transaction.");
        }

        // Set amount as needed
        if ($tx->amount > 0.00000000) {
            $tx->from_amount = $this->convert->exchangeMoney($tx->amount, $tx->currency, $tx->from_currency);
        } elseif ($tx->from_amount > 0.00000000) {
            $tx->amount = $this->convert->exchangeMoney($tx->from_amount, $tx->from_currency, $tx->currency);
        }

        // Get base amount
        $tx->base_amount = $this->convert->exchangeMoney($tx->amount, $tx->currency, $this->app->config('core.default_currency'));

        // Return
        return $tx;
    }

}

