<?php
declare(strict_types = 1);

namespace App\Transaction\Models;

use Apex\Svc\{Container, Convert, Db};
use Apex\App\Base\Model\MagicModel;
use App\Transaction\Models\{Currency, TxAccount, TxNote};
use App\Transaction\Enums\{TxStatus, TxAmountType};
use App\Transaction\Handlers\TxHandler;
use App\Transaction\Exceptions\TxValidationException;
use App\Transaction\Interfaces\AdapterInterface;
use App\Users\User;
use Apex\App\Adapters\ArmorAdapter;
use Apex\App\Base\Model\ModelIterator;
use DateTime;

/**
 * Tx Model
 */
class Tx extends MagicModel
{

    #[Inject(Container::class)]
    protected Container $cntr;

    #[Inject(Convert::class)]
    protected Convert $convert;

    #[Inject(Db::class)]
    protected Db $db;

    #[Inject(TxHandler::class)]
    private TxHandler $tx_handler;

    #[Inject(ArmorAdapter::class)]
    private ArmorAdapter $armor_adapter;

    // Properties
    private ?TxStatus $orig_status = null;
    private mixed $orig_amount = null;
    private mixed $orig_from_amount = null;
    private mixed $orig_base_amount = null;
    private ?string $orig_currency = null;
    private ?string $orig_from_currency = null;

    /**
     * Database table
     *
     * @var string
     */
    protected static string $dbtable = 'transaction';

    /**
     * Constructor
     */
    public function __construct(
        protected int $id = 0,
        protected int $account_id = 0, 
        protected string $uuid = '', 
        protected TxStatus $status = TxStatus::APPROVED,
        protected string $adapter = 'system', 
        protected int $index_id = 0, 
        protected int $reference_id = 0, 
        protected string $currency = '', 
        protected string $from_currency = '', 
        protected float | string $amount = 0.00, 
        protected float | string $from_amount = 0.00000000, 
        protected float | string $base_amount = 0.00000000, 
        protected float | string $fee = 0.00000000, 
        protected float $fee_percent = 0.00, 
        protected int $payment_id = 0, 
        protected string $ip_address = '', 
        protected string $description = '',
        protected ?DateTime $created_at = null, 
        protected ?DateTime $updated_at = null
    ) { 

        // Set orig properties
        $this->orig_status = $status;
        $this->orig_amount = $amount;
        $this->orig_from_amount = $from_amount;
        $this->orig_base_amount = $base_amount;
        $this->orig_currency = $currency;
    $this->orig_from_currency = $from_currency;

    }

    /**
     * Get account
     */
    public function getAccount():TxAccount
    {
        $obj = TxAccount::whereId($this->account_id);
        return $obj;
    }

    /**
     * Get Currency
     */
    public function getCurrency():Currency
    {
        $obj = Currency::whereFirst('abbr = %s', $this->currency);
        return $obj;
    }

    /**
     * Get Tx Note
     */
    public function getNotes(string $sort_by = '', string $sort_dir = '', int $limit = 0, int $offset = 0):ModelIterator
    {
        $result = $this->getChildren('transaction_notes.trans_id', TxNote::class, $sort_by, $sort_dir, $limit, $offset);
        return $result;
    }

    /**
     * Get adapter
     */
    public function getAdapter():AdapterInterface
    {
        $obj = $this->cntr->make($this->adapter);
        return $obj;
    }

    /**
     * Save
     */
    public function save(array $values = []):void
    {

        // Check currency
        if ($this->currency != $this->orig_currency || $this->from_currency != $this->orig_from_currency) {
            throw new TxValidationException("You can not modify the currency of a transaction.");
        }

        // Update status
        if ($this->id > 0 && $this->orig_status != $this->status) {
            $this->tx_handler->updateStatus($this->getOrigTx(), $this->status, true);
        }

        // Update amount as necessary
        $new_tx = null;
        if ($this->id > 0 && $this->amount != $this->orig_amount) {
            $new_tx = $this->tx_handler->updateAmount($this->getOrigTx(), $this->amount, TxAmountType::ACTUAL, true);
        } elseif ($this->id > 0 && $this->base_amount != $this->orig_base_amount) {
            $new_tx = $this->tx_handler->updateAmount($this->getOrigTx(), $this->base_amount, TxAmountType::BASE, true);
        } elseif ($this->id > 0 && $this->from_amount != $this->orig_from_amount) {
            $new_tx = $this->tx_handler->updateAmount($this->getOrigTx(), $this->from_amount, TxAmountType::FROM, true);
        }

        // Update property amounts, if needed
        if ($new_tx !== null) {
            $this->amount = $new_tx->amount;
            $this->from_amount = $new_tx->from_amount;
            $this->base_amount = $new_tx->base_amount;
        }

        // Update database
        parent::save($values);
    }

    /**
     * Create orig tx
     */
    private function getOrigTx():static
    {
        $new_tx = clone $this;
        $new_tx->status = $this->orig_status;
        $new_tx->amount = $this->orig_amount;
        return $new_tx;
    }

    /**
     * toDisplayArray
     */
    public function toDisplayArray(bool $include_full_details = false):array
    {

        // Initialize
        $row = $this->toArray();

        // Set variables
        $row['status'] = ucwords($this->status->value);
        $row['name'] = $this->getAdapter()->getName($this);
        $row['amount_gross'] = $this->convert->money((float) ($row['amount'] - $row['fee']), $row['currency']);
        $row['fee'] = $this->convert->money((float) $row['fee'], $row['currency']) . ' (' . $row['fee_percent'] . '%)';
        $row['amount'] = $this->convert->money((float) $row['amount'], $row['currency']);
        $row['created_at'] = $this->convert->date($row['created_at'], true);

        if ($row['payment_id'] > 0) {           
            $payment = app::make(payment::class, ['id' => (int) $row['payment_id']]);
            $row['payment_method'] = $payment->get_label();
        } else { 
            $row['payment_method'] = 'No Payment';
        }

        // Get account name
        if ($include_full_details === true) {
            $acct = $this->getAccount()->toDisplayArray();
            $row['account_name'] = $acct['display_name'];

            $user = $this->armor_adapter->getUuid($this->db, $this->uuid);
            $row['username'] = $user->getUsername();
            $row['full_name'] = $user->getFullName();
            $row['email'] = $user->getEmail();
            $row['user_created_at'] = $this->convert->date($user->getCreatedAt(), true);
        }

        // Return
        return $row;
    }

}

