<?php
declare(strict_types = 1);

namespace App\Transaction\Opus\DataTables;

use Apex\Svc\{App, Db, Convert};
use Apex\App\Interfaces\Opus\DataTableInterface;
use App\Transaction\Models\TxAccount;

/**
 * Data Table - Accounts
 */
class Accounts implements DataTableInterface
{

    // Columns
    public array $columns = [
        'label' => 'Name',
        'currency' => 'Currency',
        'rate' => 'Rate',
        'pending' => 'Pending',
        'balance' => 'Balance',
        'manage' => 'Manage',
        'generate' => 'Generate'
    ];
    public array $sortable = ['currency','pending','balance'];

    // Other variables
    public int $rows_per_page = 25;
    public bool $has_search = false;

    // Form field (left-most column)
    public string $form_field = 'checkbox';
    public string $form_name = 'account_id';
    public string $form_value = 'id'; 

    /**
     * Constructor
     *
     * All attributes within the <s:function> tag are passed to this method during instantiation, 
     * and here you can define various properties (ie. userid, type, et al) to be used in below methods.
     */
    public function __construct(
        private array $attr, 
        private App $app, 
        private Db $db,
    ) {

        // Set variables 
        $this->uuid = $attr['uuid'] ?? '';
        $this->is_crypto = isset($attr['is_crypto']) && $attr['is_crypto'] == 1 ? true : false;
        if (!isset($attr['is_crypt'])) { 
            $this->is_crypto = null;
        }
        $this->rates = $db->getHash("SELECT abbr,current_rate FROM transaction_currencies WHERE is_active = %b", true);

        // Change form field to 'none', if not admin
        if ($app->getArea() != 'admin') {
            $this->form_field = 'none';
        }

        // Remove rate column, if needed
        if (count($this->rates) < 2) {
            unset($this->columns['rate']);
        }

        // Remove generate column, if needed
        if ($this->is_crypto === false) {
            unset($this->columns['generate']);
        }

    }

    /**
     * Get total rows in data set - used for pagination.
     */
    public function getTotal(string $search_term = ''):int 
    {

        // Get total
        if ($this->is_crypto === null) {
            $total = $this->db->getField("SELECT count(*) FROM transaction_accounts WHERE uuid = %s AND is_deleted = %b", $this->uuid, false);
        } else {
            $total = $this->db->getField("SELECT count(*) FROM transaction_accounts WHERE uuid = %s AND is_deleted = %b AND is_crypto = %b", $this->uuid, false, $this->is_crypto);
        }

        // Return
        return (int) $total;
    }

    /**
     * Get rows to display on current page.
     *
     * Should return an array with each element being an associative array representing one table row.
     */ 
    public function getRows(int $start = 0, string $search_term = '', string $order_by = 'id asc'):array 
    {

        // Get accounts
        if ($this->is_crypto === null) {
            $accounts = TxAccount::where("uuid = %s AND is_deleted = %b ORDER BY $order_by LIMIT $this->rows_per_page OFFSET $start", $this->uuid, false);
        } else {
            $accounts = TxAccount::where("uuid = %s AND is_deleted = %b AND is_crypto = %b ORDER BY $order_by LIMIT $this->rows_per_page OFFSET $start", $this->uuid, false, $this->is_crypto);
        }
        $uri = '/' . $this->app->getArea() . '/financial/view_account';

        // Go through rows
        $results = [];
        foreach ($accounts as $acct) {
            $row = $acct->toDisplayArray();
            $row['manage'] = "<center><s:button href=\"$uri?account_id=$acct->id\" label=\"Manage\"></center>";

            if ($acct->is_crypto === true) {
                $row['generate'] = "<center><a href=\"javascript:openModal('bitcoin/generate_address', 'account_id=$acct->id');\" class=\"btn btn-warning btn-md\">Generate</a></center>";
            } else {
                $row['generate'] = "&nbsp;;";
            }
            $results[] = $row;
        }

        // Return
        return $results;
    }

    /**
     * Format individual row for display to browser.
     */
    public function formatRow(array $row):array
    {

        // Return
        return $row;
    }

}

