<?php
declare(strict_types = 1);

namespace App\Transaction\Opus\DataTables;

use Apex\Svc\{App, Db, Convert};
use App\Transaction\Models\{Tx, TxAccount};
use Apex\App\Interfaces\Opus\DataTableInterface;

/**
 * Data Table - TransactionList
 */
class TransactionList implements DataTableInterface
{

    // Columns
    public array $columns = [
        'created_at' => 'Date',
        'username' => 'User',
        'name' => 'Name',
        'status' => 'Status',
        'amount' => 'Amount',
        'deposit' => 'Deposit',
        'withdraw' => 'Withdraw',
        'balance' => 'Balance',
        'process' => 'Process',
        'view' => 'View'
    ];
    public array $sortable = ['created_at', 'amount', 'status'];

    // Other variables
    public int $rows_per_page = 50;
    public bool $has_search = false;

    // Form field (left-most column)
    public string $form_field = 'none';
    public string $form_name = 'transaction_list_id';
    public string $form_value = 'id'; 

    // Additional properties
    private string $where_sql;
    private array $where_args;

    /**
     * Constructor
     *
     * All attributes within the <s:function> tag are passed to this method during instantiation, 
     * and here you can define various properties (ie. userid, type, et al) to be used in below methods.
     */
    public function __construct(
        private array $attr, 
        private App $app, 
        private Db $db,
        private Convert $convert
    ) { 

        // Set variables
        $this->uuid = $attr['uuid'] ?? '';
        $this->account_id = $attr['account_id'] ?? 0;
        $this->payment_id = $attr['payment_id'] ?? 0;
        $this->adapter = $attr['adapter'] ?? '';
        $this->status = $attr['status'] ?? '';

        // Remove necessary columns
    $this->removeColumns();

        // Generate where sql
        $this->generateWhereSql();
    }

    /**
     * Remove necessary columns
     */
    private function removeColumns(): void
    {

        // Remove username column
        if ($this->uuid != '' || $this->account_id > 0) {
            unset($this->columns['username']);
        }

        // Remove status
        if ($this->status != '') {
            unset($this->columns['status']);
        }

        // Remove necessary amount columns
        $del_columns = $this->account_id == 0 || ($this->status != '' && $this->status != 'approved') ? ['deposit','withdraw','balance'] : ['amount'];
        foreach ($del_columns as $col_alias) {
            unset($this->columns[$col_alias]);
        }

        // Hide buttons as needed
        if ($this->status == 'pending_user_confirm') { 
            unset($this->columns['view']);
        } else {
            unset($this->columns['process']);
        }

    }

    /**
     * Generate where sql
     */
    private function generateWhereSql():void
    {

        // Initialize
        $sql = 'id > 0 AND ';
        $args = [];

        // Account id / uuid
        if ($this->account_id > 0) {
            $sql .= "account_id = %i AND ";
            $args[] = $this->account_id;
        } elseif ($this->uuid != '') {
            $sql .= "uuid = %s AND ";
            $args[] = $this->uuid;
        }

        // Payment id
        if ($this->payment_id > 0) {
            $sql .= "payment_id = %i AND ";
            $args[] = $this->payment_id;
        }

        // Status
        if ($this->status != '') {
            $sql .= "status = %s AND ";
            $args[] = $this->status;
        }

        // Adapter
        if ($this->adapter != '') {
            $sql .= "adapter = %s AND ";
            $args[] = $this->adapter;
        }

        // Finish up
        $this->where_sql = preg_replace("/ AND $/", "", $sql);
        $this->where_args = $args;
    }

    /**
     * Get total rows in data set - used for pagination.
     */
    public function getTotal(string $search_term = ''):int 
    {
        // Get total
        $total = $this->db->getField("SELECT count(*) FROM transaction WHERE $this->where_sql", ...$this->where_args);
        return (int) $total;
    }

    /**
     * Get rows to display on current page.
     *
     * Should return an array with each element being an associative array representing one table row.
     */ 
    public function getRows(int $start = 0, string $search_term = '', string $order_by = 'created_at desc'):array 
    {

        // Get current balance, if needed
        $balance = '0';
        if ($this->account_id > 0 && $this->status == 'approved') {
            $acct = TxAccount::whereId($this->account_id);
            $balance = (string) $acct->balance;
        }

        // Get rows
        $rows = Tx::where($this->where_sql . " ORDER BY $order_by LIMIT $this->rows_per_page OFFSET $start", ...$this->where_args);

        // Go through rows
        $results = [];
        foreach ($rows as $tx) { 

            // Get tx details
            $row = $tx->toDisplayArray();

            // View button
            $uri = '/' . $this->app->getArea() . '/financial/viewtx?txid=' . $row['id'];
            $row['view'] = "<center><s:button href=\"$uri\" label=\"View\"></center>";

            // Update balance, if needed
            if ($this->account_id > 0 && $this->status == 'approved') {

                // Update balance
                $column = bccomp((string) $tx->amount, '0', 8) == 1 ? 'deposit' : 'withdraw';
                $amount = preg_replace("/^-/", "", (string) $tx->amount);
                if (count($results) > 0) {
                    $balance = $column == 'deposit' ? bcadd($balance, $amount, 8) : bcsub($balance, $amount, 8);
                }

                // Set variables
                $row['deposit'] = $column == 'deposit' ? $this->convert->money($amount, $tx->currency, false) : "&nbsp;";
                $row['withdraw'] = $column == 'withdraw' ? $this->convert->money($amount, $tx->currency, false) : "&nbsp;";
                $row['balance'] = $this->convert->money($balance, $tx->currency, false);
            }

            // Add to results
            $results[] = $row;
        }

        // Return
        return $results;
    }

    /**
     * Format individual row for display to browser.
     */
    public function formatRow(array $row):array
    {


        // Return
        return $row;
    }

}

