<?php
declare(strict_types = 1);

namespace App\Users\Opus\DataTables;

use Apex\Svc\{App, Db};
use Apex\App\Base\Web\Utils\GraphUtils;
use Apex\App\Interfaces\Opus\DataTableInterface;

/**
 * Data Table - RegistrationReport
 */
class RegistrationReport implements DataTableInterface
{

    // Columns
    public array $columns = [
        'date' => 'Date'
    ];
    public array $sortable = ['id'];

    // Other variables
    public int $rows_per_page = 50;
    public bool $has_search = false;

    // Form field (left-most column)
    public string $form_field = 'none';
    public string $form_name = 'registration_report_id';
    public string $form_value = 'id'; 

    /**
     * Constructor
     *
     * All attributes within the <s:function> tag are passed to this method during instantiation, 
     * and here you can define various properties (ie. userid, type, et al) to be used in below methods.
     */
    public function __construct(
        private array $attr, 
        private App $app, 
        private Db $db,
        private GraphUtils $graph_utils
    ) {

        // Get period 
        $this->period = $this->app->post('period', 'day');

        // Add user groups as columns
        $this->groups = $this->db->getHash("SELECT id,name FROM user_groups ORDER BY id");
        foreach ($this->groups as $id => $name) {
            $this->columns['group' . $id] = $name;
        }
        $this->columns['total'] = 'Total';

        // Get start date
        if (!$this->start_date = $this->db->getField("SELECT created_at FROM armor_users WHERE type = 'user' ORDER BY created_at LIMIT 1")) {
            $this->start_date = '';
        }

    }

    /**
     * Get total rows in data set - used for pagination.
     */
    public function getTotal(string $search_term = ''):int 
    {

        // Check start date
        if ($this->start_date == '') { 
            return 0;
        }

        // Get total
        $total = $this->graph_utils->getTotalPeriods($this->start_date, $this->period);
        return $total;
    }

    /**
     * Get rows to display on current page.
     *
     * Should return an array with each element being an associative array representing one table row.
     */ 
    public function getRows(int $start = 0, string $search_term = '', string $order_by = 'id asc'):array 
    {

        // Get start date
        if ($start > 0) {
            $start_date = $this->db->subtractTime($this->period, $start, date('Y-m-d H:i:s'));
        } else { 
            $start_date = date('Y-m-d');
        }

        // Go through last 50 rows
        $results = [];
        for ($x=0; $x < 50; $x++) { 

            // Parse date
            list($where_sql, $display_date, $date) = $this->graph_utils->getPeriodSql('armor_users.created_at', $x, $this->period, $start_date);
            if ($this->db->getField("SELECT DATE('$this->start_date') > DATE('$date')")) { 
                break;
            }

            // Get totals
            $totals = $this->db->getHash("SELECT group_id,count(*) FROM user_profiles, armor_users WHERE user_profiles.uuid = armor_users.uuid AND $where_sql GROUP BY user_profiles.group_id"); 

            // Set row
            $row = [
                'date' => $display_date, 
                'total' => 0
            ];

            // Go through user groups
            foreach ($this->groups as $group_id => $group_name) { 
                $total = $totals[$group_id] ?? 0;
                $row['group' . $group_id] = $total;
                $row['total'] += $total;
            }
            array_push($results, $row);
        }

        // Return
        return $results;
    }

    /**
     * Format individual row for display to browser.
     */
    public function formatRow(array $row):array
    {


        // Return
        return $row;
    }

}

