<?php
declare(strict_types = 1);

namespace App\Users\Opus\DataTables;

use Apex\Svc\{App, Db, Convert};
use Apex\App\Interfaces\Opus\DataTableInterface;

/**
 * Data Table - UserList
 */
class UserList implements DataTableInterface
{

    // Columns
    public array $columns = [
        'uuid' => 'ID',
        'username' => 'Username',
        'email' => 'E-Mail',
        'group_id' => 'Group',
        'created_at' => 'Date Created',
        'manage' => 'Manage'
    ];
    public array $sortable = ['uuid', 'username', 'email', 'group_id', 'created_at'];

    // Other variables
    public int $rows_per_page = 50;
    public bool $has_search = true;

    // Form field (left-most column)
    public string $form_field = 'checkbox';
    public string $form_name = 'uuid';
    public string $form_value = 'uuid'; 

    /**
     * Constructor
     *
     * All attributes within the <s:function> tag are passed to this method during instantiation, 
     * and here you can define various properties (ie. userid, type, et al) to be used in below methods.
     */
    public function __construct(
        private array $attr, 
        private App $app, 
        private Db $db,
        private Convert $convert
    ) {
        $this->groups = $db->getHash("SELECT id,name FROM user_groups");
        $this->is_pending = isset($attr['is_pending']) && $attr['is_pending'] == 1 ? true : false;
        if ($this->is_pending === true) {
            $this->has_search = false;
        }

    }

    /**
     * Get total rows in data set - used for pagination.
     */
    public function getTotal(string $search_term = ''):int 
    {

        // Get total
        if ($search_term != '') { 
            $total = $this->db->getField("SELECT count(*) FROM armor_users u, user_profiles p WHERE u.uuid = p.uuid AND u.type = 'user' AND (u.username LIKE %ls OR u.email LIKE %ls OR p.first_name LIKE %ls OR p.last_name LIKE %ls)", $search_term, $search_term, $search_term, $search_term);
        } elseif ($this->is_pending === true) {
            $total = $this->db->getField("SELECT count(*) FROM armor_users WHERE type = 'user' AND is_pending = %b", true);
        } else { 
            $total = $this->db->getField("SELECT count(*) FROM armor_users WHERE type = 'user'");
        }

        // Return
        return (int) $total;
    }

    /**
     * Get rows to display on current page.
     *
     * Should return an array with each element being an associative array representing one table row.
     */ 
    public function getRows(int $start = 0, string $search_term = '', string $order_by = 'created_at desc'):array 
    {

        // Get rows
        if ($search_term != '') { 
        $rows = $this->db->query("SELECT u.uuid, u.username, u.email, p.first_name, p.last_name, p.group_id, u.created_at FROM armor_users u, user_profiles p WHERE u.uuid = p.uuid AND u.type = 'user' AND (u.username LIKE %ls OR u.email LIKE %ls OR p.first_name LIKE %ls OR p.last_name LIKE %ls) ORDER BY $order_by LIMIT $start,$this->rows_per_page", $search_term, $search_term, $search_term, $search_term);
        } elseif ($this->is_pending === true) {
            $rows = $this->db->query("SELECT u.uuid, u.username, u.email, p.first_name, p.last_name, p.group_id, u.created_at FROM armor_users u, user_profiles p WHERE u.uuid = p.uuid AND u.type = 'user' AND u.is_pending = %b ORDER BY $order_by LIMIT $start,$this->rows_per_page", true);
        } else { 
            $rows = $this->db->query("SELECT u.uuid, u.username, u.email, p.first_name, p.last_name, p.group_id, u.created_at FROM armor_users u, user_profiles p WHERE u.uuid = p.uuid AND u.type = 'user' ORDER BY $order_by LIMIT $start,$this->rows_per_page");
        }

        // Go through rows
        $results = [];
        foreach ($rows as $row) { 
            $results[] = $this->formatRow($row);
        }

        // Return
        return $results;
    }

    /**
     * Format individual row for display to browser.
     */
    public function formatRow(array $row):array
    {

        // Initialize
        $auth_type = $this->app->getArea() == 'admin' ? 'admin' : 'user';

        // Format
        $row['group_id'] = $this->groups[$row['group_id']] ?? 'Unknown';
        $row['created_at'] = $this->convert->date($row['created_at']);
        $row['manage'] = "<center><a href=\"javascript:openModal('users/manage', 'uuid=$row[uuid]&auth_type=$auth_type');\" class=\"btn btn-primary btn-md\">Manage</a></center>";

        // Return
        return $row;
    }

}

