<?php
declare(strict_types = 1);

namespace App\Users\Opus\Forms;

use Apex\Svc\{App, Db};
use Apex\App\Base\Web\Utils\FormBuilder;
use Apex\Armor\Policy\PolicyManager;
use App\Users\User;
use Apex\App\Interfaces\Opus\FormInterface;


/**
 * Form - Profile
 */
class Profile implements FormInterface
{

    #[Inject(PolicyManager::class)]
    private PolicyManager $policy_manager;

    #[Inject(FormBuilder::class)]
    private FormBuilder $builder;

    #[Inject(App::class)]
    private App $app;

    #[Inject(Db::class)]
    private Db $db;

    /**
     * Whether or not to pre-populate form fields with POSTed data.
     */
    public bool $allow_post_values = true;

    /**
     * Get form fields.
     */
    public function getFields(array $attr = []):array
    {

        // Initialize
        $builder = $this->builder;
        $show_full = $attr['show_full'] ?? 0;
        $show_login = $attr['show_login'] ?? 1;
        $is_secure = $attr['is_secure'] ?? 0;

        // Get armor policy
        $policy = $this->policy_manager->loadPolicy('user');
        $username_field = match($policy->getUsernameColumn()) { 
            'email' => 'E-Mail Address',
            'phone' => 'Phone Number',
            default => 'Username'
        };

        // Add basic details, if needed
        $form_fields = [];
        if ($show_full == 1 && $show_login == 0 && $this->app->getArea() == 'admin') { 
            $form_fields['sep_basic'] = $builder->seperator('Basic');
            $form_fields['group_id'] = $builder->select()->dataSource('table.user_groups.~name~.id');
            $form_fields['is_active'] = $builder->boolean()->value(true);
            $form_fields['is_frozen'] = $builder->boolean()->value(false);
            $form_fields['is_pending'] = $builder->boolean()->value(false);
            $form_fields['is_deleted'] = $builder->boolean()->value(false);
            $form_fields['email_verified'] = $builder->boolean()->value(false);
            $form_fields['phone_verified'] = $builder->boolean()->value(false);
        }

        // Add login details, if needed
        if ($show_login == 1) { 
            $form_fields['sep_login'] = $builder->seperator('Login Details');

            // Get total number of user groups
            if ($this->app->getArea() == 'admin') { 
                $groups = $this->db->getHash("SELECT id,name FROM user_groups");
            } else { 
                $groups = $this->db->getHash("SELECT id,name FROM user_groups WHERE show_in_form = 1");
            }

            // Add user group, if needed
            if (count($groups) > 1) { 
                $form_fields['group_id'] = $builder->select()->dataSource('table.user_groups.~name~.id')->label('User Group');
            }

            // Add username field
            $form_fields['username'] = $builder->textbox()->label($username_field)->required()->placeholder('Desired Username');
            $form_fields['password'] = $builder->textbox()->label('Desired Password')->type('password')->required();
            $form_fields['password2'] = $builder->textbox()->label('Confirm Password')->type('password')->required();
        }
        $form_fields['sep_profile'] = $builder->seperator('Profile');

        // Add name fields
        $name_required = $this->app->config('users.name_required') == 1 ? true : false;
        if ($this->app->config('users.name_column') == 'full') {
            $form_fields['full_name'] = $builder->textbox()->required($name_required);
        } elseif ($this->app->config('users.name_column') == 'first_last') { 
            $form_fields['first_name'] = $builder->textbox()->required($name_required);
            $form_fields['last_name'] = $builder->textbox()->required($name_required);
        }

        // Add e-mail address, if needed
        if ($policy->getUsernameColumn() != 'email') { 
            if ($is_secure == 1) {
                $form_fields['email'] = $builder->label()->label('E-Mail Address');
            } else { 
                $form_fields['email'] = $builder->textbox()->required()->label('E-Mail Address')->dataType('email');
            }
        }

        // Add phone, if needed
        if ($policy->getRequirePhone() != 'none') {
            if ($is_secure == 1) {
                $form_fields['phone'] = $builder->label()->label('Phone Number');
            } else {
                $require_phone = $policy->getRequirePhone() == 'require' ? true : false;
                $form_fields['phone'] = $builder->phone()->required($require_phone);
            }
        }

        // Add profile fields
        if ($show_full == 1) { 

            // Add language, timezone, currency
            $form_fields['sep_addl'] = $builder->seperator('Additional');
            $form_fields['language'] = $builder->select()->required()->dataSource('stdlist.language')->value($this->app->config('core.default_language'));
            $form_fields['timezone'] = $builder->select()->required()->dataSource('stdlist.timezone')->value($this->app->config('core.default_timezone'));
            $form_fields['currency'] = $builder->select()->required()->dataSource('stdlist.currency')->value($this->app->config('core.default_currency'));

            // Add additional profile fields
            $rows = $this->db->query("SELECT * FROM user_profile_fields WHERE show_in_form = %b ORDER BY order_num", true);
            foreach ($rows as $row) {
                $method = $row['form_field'];
                $required = $row['is_required'] == 1 ? true : false;
                $form_fields[$row['alias']] = $builder->$method()->label($row['name'])->required($required);
            }
        }

        // Add submit button
        $name = $this->app->getArea() == 'admin' ? 'User' : 'Account';
        $form_fields['submit'] = $builder->createOrUpdateButton($name, $attr);

        // Return
        return $form_fields;
    }

    /**
     * Get record
     */
    public function getRecord(string $record_id):array
    {

        // Get row
        if (!$user = User::loadUuid($record_id)) { 
            return [];
        }

        // Return
        return $user->toArray();
    }

    /**
     * Validate
     */
    public function validate(array $attr = []):bool
    {

        // Return
        return true;
    }

}

