<?php
declare(strict_types = 1);

namespace App\Users;

use Apex\Svc\{Db, Di, Dispatcher, MessageRequest, App};
use App\Users\Models\Note;
use Apex\Armor\User\ArmorUser;
use Apex\App\Interfaces\UserInterface;

/**
 * User model
 */
class User extends ArmorUser implements UserInterface
{

    #[Inject(App::class)]
    private App $app;

    #[Inject(Db::class)]
    private Db $db;

    // Properties
    private array $extended = [];

    /**
     * Constructor
     */
    public function __construct(
        private string $first_name, 
        private string $last_name, 
        private int $group_id, 
        private string $language, 
        private string $currency, 
        private string $timezone,
        private string $extra_profile = ''
    ) { 

    }

    /**
     * Get first name
     */
    public function getFirstName():string
    {
        return $this->first_name;
    }

    /**
     * Get last name
     */
    public function getLastName():string
    {
        return $this->last_name;
    }

    /**
     * Get full name
     */
    public function getFullName():string
    {
        return $this->first_name . ' ' . $this->last_name;
    }

    /**
     * Get language
     */
    public function getLanguage():string
    {
        return $this->language;
    }

    /**
     * Get currency
     */
    public function getCurrency():string
    {
        return $this->currency;
    }

    /**
     * Return timezone
     */
    public function getTimezone():string
    {
        return $this->timezone;
    }

    /**
     * Get group id
     */
    public function getGroupId():int
    {
        return $this->group_id;
    }

    /**
     * To array
     */
    public function toArray():array
    {

        // Set variables
        $vars = [
            'uuid' => $this->uuid, 
            'is_active' => $this->is_active === true ? 1 : 0, 
            'is_deleted' => $this->is_deleted === true ? 1 : 0, 
            'is_pending' => $this->is_pending === true ? 1 : 0, 
            'is_frozen' => $this->is_frozen === true ? 1 : 0,
            'has_username' => ($this->username == '' ? 0 : 1), 
            'has_password' => ($this->password == '' ? 0 : 1), 
            'has_email' => ($this->email == '' ? 0 : 1), 
            'has_phone' => ($this->phone == '' ? 0 : 1),  
            'email_verified' => $this->email_verified === true ? 1 : 0, 
            'phone_verified' => $this->phone_verified === true ? 1 : 0, 
            'username' => $this->username, 
            'email' => $this->email, 
            'phone' => $this->phone,
            'phone_country' => $this->getPhoneCountryCode(), 
            'phone_national' => $this->getPhoneNational(), 
            'phone_formatted' => $this->getPhoneFormatted(), 
            'full_name' => $this->getFullName(), 
            'first_name' => $this->first_name, 
            'last_name' => $this->last_name, 
            'two_factor_type' => $this->two_factor_type, 
            'two_factor_frequency' => $this->two_factor_frequency, 
            'group_id' => $this->group_id,
            'timezone' => $this->timezone,
            'language' => $this->language,
            'currency' => $this->currency,
            'created_at' => $this->created_at, 
            'updated_at' => $this->updated_at,
            'reg_country' => $this->reg_country,
            'reg_province' => $this->reg_province_name,
            'reg_city' => $this->reg_city,
            'reg_ip' => $this->reg_ip,
            'reg_user_agent' => $this->reg_user_agent,
            'require_2fa' => $this->two_factor_type
        ];

        // Return
        return $vars;
    }

    /**
     * Get display array
     */
    public function toDisplayArray():array
    {

        // Get profile
        $vars = $this->toArray();

        // Set booleans
        $booleans = [
            'is_active',
            'is_pending',
            'is_frozen',
            'email_verified',
            'phone_verified'
        ];

        // Format booleans
        foreach ($booleans as $bool) { 
            $vars[$bool] = $vars[$bool] == 1 ? 'Yes' : 'No';
        }

        // Get group name
        $vars['group_name'] = $this->db->getField("SELECT name FROM user_groups WHERE id = %i", $this->group_id);

        // Return
        return $vars;
    }

    /**
     * Get extended profile
     */
    public function getExtended(string $package):mixed
    {

        // Check if already loaded
        if (isset($this->extended[$package])) { 
            return $this->extended[$package];
        }

        // Send message
        $msg = new MessageRequest('users.profiles.load', $this->uuid);
        $msg->setTarget($package);
        $dispatcher = Di::get(Dispatcher::class);
        $res = $dispatcher->dispatch($msg)->getResponse($package);

        // Set and return
        $this->extended[$package] = $res;
        return $res;
    }

    /**
     * Add note
     */
    public function addNote(string $note, ?strint $from = null):Note
    {

        // Get from, if needed
            if ($from === null) {
            $from = $this->app->getUuid();
        }

            // Add to database
        $note = Note::insert([
            'uuid' => $this->uuid,
            'added_by' => $from,
            'note' => $note
        ]);

        // Return
        return $note;
    }

    /**
     * Load by uuid
     */
    public static function loadUuid(string $uuid, bool $is_deleted = false):?static
    {

        // Get from db
        $db = Di::get(Db::class);
        if (!$user = $db->getObject(User::class, "SELECT * FROM user_profiles, armor_users WHERE armor_users.uuid = %s AND user_profiles.uuid = armor_users.uuid AND armor_users.is_deleted = %b", $uuid, $is_deleted)) { 
            return null;
        }

        // Return
        return $user;
    }

    /**
     * Load by username
     */
    public static function loadUsername(string $username, bool $is_deleted = false):?static
    {

        // Get from db
        $db = Di::get(Db::class);
        if (!$user = $db->getObject(User::class, "SELECT * FROM user_profiles, armor_users WHERE armor_users.username = %s AND user_profiles.uuid = armor_users.uuid AND armor_users.is_deleted = %b", $username, $is_deleted)) { 
            return null;
        }

        // Return
        return $user;
    }




}

