<?php
declare(strict_types = 1);

namespace App\Users;

use Apex\Svc\{App, Db, Dispatcher, MessageRequest, View};
use App\Users\User;
use App\Users\Helpers\UserNotifications;
use App\Webapp\Notifications\Notifications;
use Apex\Armor\Armor;
use Apex\Armor\User\Validator;
use Apex\Armor\Auth\AuthSession;
use Apex\Armor\Auth\Operations\Phone;
use Apex\Armor\Exceptions\ArmorProfileValidationException;
use Apex\App\Attr\Inject;

/**
 * Manage user profiles
 */
class UserProfiles
{

    #[Inject(App::class)]
    private App $app;

    #[Inject(Db::class)]
    private Db $db;

    #[Inject(Armor::class)]
    private Armor $armor;

    #[Inject(Dispatcher::class)]
    private Dispatcher $dispatcher;

    #[Inject(View::class)]
    private View $view;

    #[Inject(Validator::class)]
    private Validator $validator;

    #[Inject(Notifications::class)]
    private Notifications $notifications;

    /**
     * Check auth
     */
    public function checkAuth():?AuthSession
    {
        if (!$session = $this->armor->checkAuth()) { 
            return null;
        }

        // Set uuid, and return
        $this->app->setUuid($session->getUuid());
        return $session;
    }

    /**
     * Create admin
     */
    public function create(array $post = [], bool $auto_login = false):?User
    {

        // Initialize
        if (count($post) == 0) { 
            $post = $this->app->getAllPost();
        }

        // Format phone and username
        $phone = Phone::get($post);
        if ($this->app->config('users.lowercase_username') == 1) {
            $post['username'] = strtolower($post['username']);
        }

        // Get first / last name
        list($first_name, $last_name) = ['', ''];
        if ($this->app->config('users.name_column') == 'full' && isset($post['full_name']) && str_contains($post['full_name'], " ")) {
            list($first_name, $last_name) = explode(" ", $post['full_name'], 2);
        } else if ($this->app->config('users.name_column') == 'full') {
            $first_name = $post['full_name'] ?? '';
        } elseif ($this->app->config('users.name_column') == 'first_last') {
            $first_name = $post['first_name'];
            $last_name = $post['last_name'];
        }

        // Validate user
        try {
            $this->validator->validate(
                '', 
                ($post['password'] ?? ''), 
                ($post['username'] ?? ''),
                ($post['email'] ?? ''),
                $phone
            );
        } catch(ArmorProfileValidationException $e) {
            $this->view->addCallout($e->getMessage(), 'error');
            return null;
        }

        // Dispatch message
        $msg = new MessageRequest('users.profiles.preCreate', $post);
        $this->dispatcher->dispatch($msg);

        // Begin transaction
        $this->db->beginTransaction();

        // Create via Armor
        $user = $this->armor->createUser('', $post['password'], $post['username'], $post['email'], $phone, 'user', null, $auto_login);

        // Get group id
        $group_id = $this->app->post('group_id', 0);
        if ($group_id == 0) {
            $group_id = $this->app->config('users.default_group');
        }

        // Add to database
        $this->db->insert('user_profiles', [
            'uuid' => $user->getUuid(), 
            'first_name' => $first_name,
            'last_name' => $last_name,
            'group_id' => $group_id
        ]);

        // Commit, and load user
        $this->db->commit();
        $user = $this->loadUuid($user->getUuid());

        //  Send e-mail notifications
        $condition = [
            'action' => 'create',
            'group_id' => $group_id
        ];
        $this->notifications->process(UserNotifications::class, $user->getUuid(), $condition);

        // Dispatch message
        $msg = new MessageRequest('users.profiles.postCreate', $user->getUuid());
        $this->dispatcher->dispatch($msg);

        // Return
        return $user;
    }

    /**
     * Load by uuid
     */
    public function loadUuid(string $uuid, bool $is_deleted = false):?User
    {

        // Get from db
        if (!$user = $this->db->getObject(User::class, "SELECT * FROM user_profiles, armor_users WHERE user_profiles.uuid = %s AND user_profiles.uuid = armor_users.uuid AND armor_users.is_deleted = %b", $uuid, $is_deleted)) { 
            return null;
        }

        // Return
        return $user;
    }

    /**
     * Load by username
     */
    public function loadUsername(string $username, bool $is_deleted = false):?User
    {

        // Get from db
        if (!$user = $this->db->getObject(User::class, "SELECT * FROM user_profiles, armor_users WHERE armor_users.username = %s AND user_profiles.uuid = armor_users.uuid AND armor_users.is_deleted = %b", $username, $is_deleted)) { 
            return null;
        }

        // Return
        return $user;
    }

    /**
     * Update
     */
    public function update(User $user, array $post = []):void
    {

        // Check for demo
        if ($this->isDemo() && $user->getUsername() == 'demo') {
            return;
        }

        // Initialize
        if (count($post) == 0) { 
            $post = $this->app->getAllPost();
        }
        $is_admin = $this->app->getArea() == 'admin' ? true : false;

        // Update e-mail address
        $email = $post['email'] ?? null;
        if ($email !== null && $email != $user->getEmail()) { 
            $user->updateEmail($email, $is_admin);
        }

        // Update phone
        $phone = $post['phone'] ?? '';
        if ($phone != '' && $phone != $user->getPhoneNational()) { 
            $phone = '+' . $post['phone_country'] . $phone;
            $user->updatePhone($phone, $is_admin);
        }

        // Set vars
        $vars = [
            'first_name',
            'last_name',
            'language',
            'timezone',
            'currency'
        ];

        // Add name fields to updates
        list($first_name, $last_name) = $this->parseFullName($post);
        $post['first_name'] = $first_name;
        $post['last_name'] = $last_name;

        // Create updates
        $updates = [];
        foreach ($vars as $var) { 
            if (!isset($post[$var])) { 
                continue;
            }
            $updates[$var] = $post[$var];
        }

        // Update database, if needed
        if (count($updates) > 0) { 
            $this->db->update('user_profiles', $updates, "uuid = %i", $user->getUuid()); 
        }

        // Update as admin, if needed
        if ($is_admin === true) { 
            $this->updateAsAdmin($user, $post);
        }
    }

    /**
     * Update as administrator
     */
    public function updateAsAdmin(User $user, array $post):void
    {

        // Check for demo
        if ($this->isDemo() && $user->getUsername() == 'demo') {
            return;
        }

        // Update username
        $username = $post['username'] ?? null;
        if ($username !== null && $username != $user->getUsername()) { 
            $user->updateUsername($username);
        }

        // Update group_id, if needed
        $group_id = $post['group_id'] ?? null;
        if ($group_id !== null && $user->getGroupId() != $group_id) { 
            $this->db->query("UPDATE user_profiles SET group_id = %i WHERE uuid = %s", $post['group_id'], $user->getUuid());
        }

        // Set vars
        $vars = [
            'is_active',
            'is_frozen',
            'is_pending',
            'is_deleted',
            'email_verified',
            'phone_verified'
        ];

        // Go through vars
        $updates = [];
        foreach ($vars as $var) { 
            if (!isset($post[$var])) { 
                continue;
            }
            $updates[$var] = $post[$var];
        }

        // Update database, if needed
        if (count($updates) > 0) { 
            $this->db->update('armor_users', $updates, 'uuid = %s', $user->getUuid());
        }

    }

    /**
     * Remove
     */
    public function remove(User $user):void
    {

        // Check for demo
        if ($this->isDemo() === true && $user->getUsername() == 'demo') {
            throw new \Exception("Unable to delete the demo account.");
        }

        // Delete
        $this->db->query("DELETE FROM user_profiles WHERE uuid = %s", $user->getUuid());
        $this->armor->removeUuid($user->getUuid());
    }

    /**
     * Check if demo
     */
    private function isDemo():bool
    {
        $install_image = $this->app->config('core.install_image');
        return $install_image == 'demo' ? true : false;
    }

    /**
     * Parse full name
     */
    private function parseFullName(array $post):array
    {

        // First and last name
        if ($this->app->config('users.name_column') == 'first_last') {
            return [$post['first_name'], $post['last_name']];
        } elseif ($this->app->config('users.name_column') == 'none') {
            return ['', ''];
        }

        // Check full name
        if (preg_match("/^(.+?)\s(.+)$/", $post['full_name'], $m)) {
            return [$m[1], $m[2]];
        } else {
            return [$post['full_name'], ''];
        }

    }

}

