<?php
declare(strict_types = 1);

namespace App\Webapp;

use Apex\Svc\{App, Db, View};
use Apex\App\Adapters\ArmorAdapter;
use Apex\Armor\Armor;
use Apex\Armor\User\Validator;
use Apex\Armor\Auth\AuthSession;
use Apex\Armor\Auth\Operations\Phone;
use Apex\Armor\Exceptions\ArmorProfileValidationException;
use App\Webapp\Admin;

/**
 * Admin profiles
 */
class AdminProfiles
{

    #[Inject(App::class)]
    private App $app;

    #[Inject(Db::class)]
    private Db $db;

    #[Inject(View::class)]
    private View $view;

    #[Inject(Armor::class)]
    private Armor $armor;

    #[Inject(ArmorAdapter::class)]
    private ArmorAdapter $armor_adapter;

    #[Inject(Validator::class)]
    private Validator $validator;

    /**
     * Check auth
     */
    public function checkAuth():?AuthSession
    {
        if (!$session = $this->armor->checkAuth('admin')) { 
            return null;
        }

        // Set uuid, and return
        $this->app->setUuid($session->getUuid());
        $this->app->setSession($session);
        return $session;
    }

    /**
     * Create admin
     */
    public function create(bool $auto_login = true):?Admin
    {

        // Initialize
        $app = $this->app;
        $phone = Phone::get($app->getAllPost());

        // Validate user
        try {
            $this->validator->validate(
                '', 
                $this->app->post('password'),
                $this->app->post('username'),
                $this->app->post('email'),
                $phone,
                'admin'
            );
        } catch(ArmorProfileValidationException $e) {
            $this->view->addCallout($e->getMessage(), 'error');
            return null;
        }

        // Begin transaction
        $this->db->beginTransaction();

        // Create via Armor
        $user = $this->armor->createUser('', $app->post('password'), $app->post('username'), $app->post('email'), $phone, 'admin', null, $auto_login);

        // Add to database
        $this->db->insert('admin', [
            'uuid' => $user->getUuid(), 
            'full_name' => $app->post('full_name'),
            'timezone' => $app->post('timezone'),
            'language' => $app->post('language')
        ]);

        // Commit, and return
        $this->db->commit();
        return $this->armor->getUuid($user->getUuid());
    }

    /**
     * Update administrator
     */
    public function update(Admin $user, array $post = []):bool
    {

        // Check for demo
        if ($this->isDemo() && $user->getUsername() == 'admin') {
            return true;
        }

        // Initialize
        if (count($post) == 0) { 
            $post = $this->app->getAllPost();
        }

        // Update e-mail address
        $email = $post['email'] ?? null;
        if ($email !== null && $email != $user->getEmail()) { 
            $user->updateEmail($email, true);
        }

        // Update phone
        $phone = $post['phone'] ?? '';
        if ($phone != '' && $phone != $user->getPhoneNational()) { 
            $phone = '+' . $post['phone_country'] . $phone;
            $user->updatePhone($phone, true);
        }

        // Set vars
        $vars = [
            'full_name',
            'language',
            'timezone'
        ];

        // Create updates
        $updates = [];
        foreach ($vars as $var) { 
            if (!isset($post[$var])) { 
                continue;
            }
            $updates[$var] = $post[$var];
        }

        // Update database, if needed
        if (count($updates) > 0) { 
            $this->db->update('admin', $updates, "uuid = %i", $user->getUuid()); 
        }

        // Return
        return true;
    }

    /**
     * Load uuid
     */
    public function loadUuid(string $uuid):?Admin
    {
        return $this->armor_adapter->getUuid($this->db, $uuid);
    }

    /**
     * Load username
     */
    public function loadUsername(string $username):?Admin
    {

        // Check for uuid
        if (!$uuid = $this->db->getField("SELECT uuid FROM armor_users WHERE type = 'admin' AND username = %s", $username)) {
            return null;
        }

        // Return
        return $this->loadUuid($uuid);
    }

    /**
     * Delete
     */
    public function delete(string $uuid):void
    {

        // Check for demo
        if ($this->isDemo() && $user->getUsername() == 'admin') {
            return;
        }

        // Delete
        $this->db->query("DELETE FROM armor_users WHERE uuid = %s", $uuid);
        $this->db->query("DELETE FROM cms_menus_acl WHERE area = 'admin' AND uuid = %s", $uuid);
    }

    /**
     * Create select options
     */
    public function createSelectOptions(string $selected = '', bool $add_prefix = false):string
    {

        // Go through admins
        $options = '';
        $rows = $this->db->query("SELECT admin.*, armor_users.* FROM admin,armor_users WHERE admin.uuid = armor_users.uuid ORDER BY admin.uuid");
        foreach ($rows as $row) { 
            $chk = $selected == $row['uuid'] ? 'selected="selected"' : '';
            $name = $add_prefix === true ? 'Admin: ' : '';
            $name .= $row['full_name'] . ' (' . $row['username'] . ' / ' . $row['email'] . ')';
            $options .= "<option value=\"$row[uuid]\" $chk>$name</option>";
        }

        // Return
        return $options;
    }

    /**
     * Check if demo
     */
    private function isDemo():bool
    {
        $install_image = $this->app->config('core.install_image');
        return $install_image == 'demo' ? true : false;
    }

    /**
     * Update menu acl
     */
    public function updateMenuAcl(string $uuid, array $menus):void
    {

        // Delete existing
        $this->db->query("DELETE FROM cms_menus_acl WHERE area = 'admin' AND uuid = %s", $uuid);


        // GO through menus
        foreach ($menus as $menu) {

            // Get parent and alias
        list($parent, $alias) = [$menu, ''];
            if (str_contains($menu, '.')) {
                list($parent, $alias) = explode('.', $menu, 2);
            }

            // Add to database
            $this->db->insert('cms_menus_acl', [
                'area' => 'admin',
                'uuid' => $uuid,
                'parent' => $parent,
                'alias' => $alias
            ]);
        }

    }

}


