<?php
declare(strict_types = 1);

namespace App\Webapp\Opus\Crontabs;

use Apex\Svc\{Emailer, Convert, Container, App};
use App\Webapp\Models\{EmailQueue, EmailLink};
use App\Webapp\AdminProfiles;
use App\Webapp\Notifications\Notifications;
use Apex\Mercury\Email\EmailMessage;
use App\Users\Exceptions\UserNotExistsException;
use Apex\App\Interfaces\Opus\CrontabInterface;
use DateTime;

/**
 * Crontab job - BroadcastEmail
 */
class BroadcastEmail implements CrontabInterface
{

    #[Inject(App::class)]
    private App $app;

    #[Inject(Container::class)]
    private Container $cntr;

    #[Inject(Emailer::class)]
    private Emailer $emailer;

    #[Inject(Convert::class)]
    private Convert $convert;

    #[Inject(AdminProfiles::class)]
    private AdminProfiles $admin_profiles;

    #[Inject(Notifications::class)]
    private Notifications $notifications;

    /**
     * Whether or not to automatically execute this crontab job.
     */
    public bool $auto_run = true;

    /**
     * How often to run this crontab job.  
     * Must be formatted as the period letter followed by the number.
     *
     * For the period, I = Minute, H = Hour, D = Day, W = Week, M = Month, Q = Quarter, Y = Year
     *
     * For example, I30 = every 30 minutes, H3 = every 3 hours, W2 = every 2 weeks, et al.
     */
    public string $interval = 'I5';

    /**
     * Description of the crontab job
     */
    public string $description = 'Broadcast Queued E-Mail Messages';

    /**
     * Process the crontab job
     */
    public function process():void
    {

        // GO through queued e-mails
        $rows = EmailQueue::where('is_pending = %b AND send_date < now()', true);
        foreach ($rows as $row) {
            $this->processQueue($row);
        }

    }

    /**
     * Process e-mail queue
     */
    private function processQueue(EmailQueue $queue):void
    {

        // Get sender
        if (!$sender = $this->admin_profiles->loadUuid($queue->sender)) {
            throw new UserNotExistsException("No admin exists with the uuid, " . $queue->sender);
        }

        // Load adapter
        $adapter = $this->cntr->make($queue->adapter);
        $condition = json_decode(base64_decode($queue->condition_vars), true) ?? [];

        // Get iterator
        $iterator = $adapter->getBroadcastIterator($queue->total_sent, $condition);

        // Get start time
        $start_time = null;
        if ($queue->started_at === null) {
            $start_time = new DateTime();
            $queue->started_at = $start_time;
            $queue->save();
        }

        // Go through recipients
        foreach ($iterator as $model) {

            // Get merge vars
            $recip = $adapter->getBroadcastRecipient($model); 
            $merge_vars = $this->notifications->getMergeVars($adapter, $recip->user, $recip->data);
            $merge_vars['msghash'] = $queue->mhash;

            // Personalize message
            list($subject, $text, $html) = [$queue->subject, $queue->text_message, $queue->html_message];
            foreach ($merge_vars as $key => $value) {
                if ($value instanceof DateTime) {
                    $value = $value->format('Y-m-d H:i:s');
                }
                $subject = str_replace("~$key~", (string) $value, $subject);
                $text = str_replace("~$key~", (string) $value, $text);
                $html = str_replace("~$key~", (string) $value, $html);
            }

            // Add tracking pixel
            if ($html != '') {
                $html = $this->addTrackingPixel($html, $queue->mhash, $recip->tracking_id);
            }

            // Parse links
            $text = $this->parseLinks($text, $queue->mhash, $recip->tracking_id, false);
            $html = $this->parseLinks($html, $queue->mhash, $recip->tracking_id, true);

            // Set e-mail message
            $email = new EmailMessage(
                to_email: $recip->contact->getEmail(),
                to_name: $recip->contact->getName(),
                from_email: $sender->getEmail(),
                from_name: $sender->getFullName(),
                reply_to: $queue->reply_to,
                subject: $subject,
                text_message: $text,
                html_message: $html
            );

            // Add attachment, if needed
            if ($queue->attach_contents != '') {
                $email->addAttachment($queue->attach_filename, base64_decode($queue->attach_contents));
            }

            // SEnd e-mail
            if (!$this->emailer->send($email)) {
                throw new \Exception("Unable to send e-mail to address, " . $recip->contact->getEmail());
            }

            // Update queue
            $queue->total_sent++;
            $queue->save();
        }

        // Set times
        $queue->completed_at = new DateTime();
        $start_time = $queue->started_at->format('Y-m-d H:i:s');
        $end_time = $queue->completed_at->format('Y-m-d H:i:s');

        // Send summary
        $summary = "\nBroadcasting of the below e-mail message to all necessary recipients has been completed.  Below gives a brief summary:\n\n";
        $summary .= "    Subject: $queue->subject\n";
        $summary .= "    Total Sent:  $queue->total_sent\n";
        $summary .= "    Start Time: " . $this->convert->date($start_time, true) . "\n";
        $summary .= "    Emd Time: " . $this->convert->date($end_time, true) . "\n";
        $summary .= "\n-- END --\n\n";

        // Send e-mail
        $email = new EmailMessage(
            to_email: $queue->sent_by,
            from_email: $sender->getEmail(),
            from_name: $sender->getFullName(),
            subject: '[SUMMARY] ' . $queue->subject,
            text_message: $summary
        );

        // Send summary
        $this->emailer->send($email);

        // Update queue
        $queue->is_pending = false;
        $queue->save();
    }

    /**
     * Add tracking pixel
     */
    private function addTrackingPixel(string $html, string $mhash, string $tracking_id):string
    {

        // ADd to HTML
        $mhash .= ':' . $tracking_id;
        $pixel_tag = '<img src="https://' . $this->app->config('core.domain_name') . '/msgopened/' . $mhash . '.png" border="0" width="1px" height="1px" />';
        $html .= $pixel_tag;

        // Return
        return $html;
    }

    /**
     * Parse links
     */
    private function parseLinks(string $text, string $mhash, string $tracking_id, bool $is_html = true):string
    {

        // GO through links
        preg_match_all("/\[url:(.+?)\]/si", $text, $link_match, PREG_SET_ORDER);
        foreach ($link_match as $match) {
            $url = strtolower($match[1]);
            $link_hash = dechex(crc32($url));

            // Get link url
            if (!$link = EmailLink::whereFirst('link_hash = %s', $link_hash)) {
                $link = EmailLink::insert([
                    'link_hash' => $link_hash,
                    'url' => $url
                ]);
            }

            // Get new text
            $link_hash .= ':' . $mhash . ':' . $tracking_id;
            $link_url = 'https://' . $this->app->config('core.domain_name') . '/msgclicked/' . $link_hash;
            $text = str_replace($match[0], $link_url, $text);
        }

        // Return
        return $text;
    }

}


