<?php
declare(strict_types = 1);

namespace App\Webapp\Opus\DataTables;

use Apex\Svc\{App, Db, Convert};
use Apex\App\Interfaces\Opus\DataTableInterface;
use App\Users\User;

/**
 * Data Table - Alerts
 */
class Alerts implements DataTableInterface
{

    // Columns
    public array $columns = [
        'sender' => 'Sender',
        'title' => 'subject',
        'created_at' => 'Date',
        'link' => 'Link',
        'view' => 'View'
    ];
    public array $sortable = ['title', 'created_at'];

    // Other variables
    public int $rows_per_page = 50;
    public bool $has_search = true;

    // Form field (left-most column)
    public string $form_field = 'checkbox';
    public string $form_name = 'alert_id';
    public string $form_value = 'id'; 

    /**
     * Constructor
     *
     * All attributes within the <s:function> tag are passed to this method during instantiation, 
     * and here you can define various properties (ie. userid, type, et al) to be used in below methods.
     */
    public function __construct(
        private array $attr, 
        private App $app, 
        private Db $db,
        private Convert $convert
    ) { 

        // Set variables
        $this->type = $attr['type'] ?? 'alert';
        $this->uuid = $attr['uuid'];


    }

    /**
     * Get total rows in data set - used for pagination.
     */
    public function getTotal(string $search_term = ''):int 
    {

        // Get total
        if ($search_term != '') { 
            $total = $this->db->getField("SELECT count(*) FROM alerts WHERE uuid = %s AND type = %s AND (title LIKE %ls OR contents LIKE %ls)", $this->uuid, $this->type, $search_term, $search_term);
        } else { 
            $total = $this->db->getField("SELECT count(*) FROM alerts WHERE uuid = %s AND type = %s", $this->uuid, $this->type);
        }

        // Return
        return (int) $total;
    }

    /**
     * Get rows to display on current page.
     *
     * Should return an array with each element being an associative array representing one table row.
     */ 
    public function getRows(int $start = 0, string $search_term = '', string $order_by = 'created_at desc'):array 
    {

        // Get rows
        if ($search_term != '') { 
            $rows = $this->db->query("SELECT * FROM alerts WHERE uuid = %s AND type = %s AND (title LIKE %ls OR contents LIKE %ls) ORDER BY $order_by LIMIT $start,$this->rows_per_page", $this->uuid, $this->type, $search_term, $search_term);
        } else { 
            $rows = $this->db->query("SELECT * FROM alerts WHERE uuid = %s AND type = %s ORDER BY $order_by LIMIT $start,$this->rows_per_page", $this->uuid, $this->type);
        }

        // Go through rows
        $results = [];
        foreach ($rows as $row) { 
            $results[] = $this->formatRow($row);
        }

        // Return
        return $results;
    }

    /**
     * Format individual row for display to browser.
     */
    public function formatRow(array $row):array
    {

        // Get sender
        $sender = User::loadUuid($row['sender']);

        // Format
        $row['sender'] = $sender->getFullName() . ' (' . $sender->getUsername() . ')';
        $row['created_at'] = $this->convert->date($row['created_at'], true);

        // Link
        $row['link'] = '';
        if ($row['url'] != '') {
            $row['link'] = "<center><s:button href=\"$row[url]\" label=\"Link\"></center>";
        }

        // View button
        $uri = '/' . $this->app->getArea() . "/inbox_view?type=" . $this->type . "&alert_id=$row[id]";
        $row['view'] = "<center><s:button href=\"$uri\" label=\"View\"></center>";

        // Add bold if unread
        if ($row['is_read'] == 0) {
            $row['sender'] = "<b>" . $row['sender'] . "</b>";
            $row['title'] = "<b>" . $row['title'] . "</b>";
            $row['created_at'] = "<b>" . $row['created_at'] . "</b>";
        }

        // Return
        return $row;
    }

}

