<?php
declare(strict_types = 1);

namespace Views\admin\communicate;

use Apex\Svc\{View, App, Emailer, Container, SmsClient};
use Apex\App\Adapters\ArmorAdapter;
use Apex\Mercury\Email\EmailMessage;
use App\Webapp\Notifications\{Notifications, NotificationOptions};
use App\Webapp\Models\EmailNotification;
use App\Webapp\AdminProfiles;
use App\Users\User;
use App\Users\Exceptions\UserNotExistsException;

/**
 * Render the template.
 */
class notify
{

    #[Inject(Container::class)]
    private Container $cntr;

    #[Inject(Emailer::class)]
    private Emailer $emailer;

    #[Inject(SmsClient::class)]
    private SmsClient $sms_client;

    #[Inject(ArmorAdapter::class)]
    private ArmorAdapter $armor_adapter;

    #[Inject(Notifications::class)]
    private Notifications $notifications;

    /**
     * Render
     */
    public function render(View $view, App $app, NotificationOptions $options, AdminProfiles $admin_profiles):void
    {

        // Create options
        $view->assign('notification_options', $options->create());
        $view->assign('admin_options', $admin_profiles->createSelectOptions());

        // Send e-mail message
        if ($app->getAction() == 'send') {

            // Get user
            $uuid = $app->post('user_id');
            if (!$user = User::loadUuid($uuid)) {
                throw new UserNotExistsException("No user exists with the uuid, $uuid");
            }

            // Send e-mail
            $method = $app->post('notification') == 'custom' ? 'sendCustomEmail' : 'sendSavedEmail';
            $this->$method($user, $app);

            // Call out
            $view->addCallout('Successfully sent e-mail message to user, ' . $user->getUsername());

        // Send SMS message
        } elseif ($app->getAction() == 'send_sms') {
            $user = $this->sendSMS($app);
            $view->addCallout('Successfully send SMS message to the user, ' . $user->getUsername());
        }

    }

    /**
     * Send saved e-mail
     */
    private function sendSavedEmail(User $user, App $app):void
    {

        // Get notification
        if (!$email = EmailNotification::whereId($app->post('notification'))) {
            throw new \Exception("E-mail notification does not exist with id#, " . $app->post('notification'));
        }

        // Send message
        $controller = $this->cntr->make($email->controller);
        $this->notifications->send($controller, $email, $user, []);
    }

    /**
     * Send custom e-mail
     */
    private function sendCustomEmail(User $user, App $app):void
    {

        // Get sender
        $admin = $this->armor_adapter->getUuid($db, $app->post('sender'));

        // Create custom message
        $email = $this->cntr->make(EmailMessage::class, [
            'to_email' => $user->getEmail(),
            'to_name' => $user->getFullName(),
            'from_email' => $admin->getEmail(),
            'from_name' => $admin->getFullName(),
            'subject' => $app->post('subject'),
            'text_message' => $app->post('message')
        ]);

        // Add attachment, if needed
        if ($app->hasFile('attachment')) {
            $contents = $app->fileContents('attachment');
            $email->addAttachment($app->fileName('attachment'), $contents);
        }

        // Send e-mail
        $this->emailer->send($email);
    }

    /**
     * Send SMS message
     */
    private function sendSMS(App $app):User
    {

        // Get user
        $uuid = $app->post('sms_user_id');
        if (!$user = User::loadUuid($uuid)) {
            throw new UserNotExistsException("No user exists with the uuid, $uuid");
        }

        // Send SMS
        $this->sms_client->send($user->getPhone(), $app->post('sms_message'));
        return $user;
    }

}


